@@ -1,21 +1,8 @@
-// MediaInfo_Config_MediaInfo - Configuration class
-// Copyright (C) 2005-2012 MediaArea.net SARL, Info@MediaArea.net
-//
-// This library is free software: you can redistribute it and/or modify it
-// under the terms of the GNU Library General Public License as published by
-// the Free Software Foundation, either version 2 of the License, or
-// any later version.
-//
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-// GNU Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public License
-// along with this library. If not, see <http://www.gnu.org/licenses/>.
-//
-//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
-//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
+/*  Copyright (c) MediaArea.net SARL. All Rights Reserved.
+ *
+ *  Use of this source code is governed by a BSD-style license that can
+ *  be found in the License.html file in the root of the source tree.
+ */
 
 //---------------------------------------------------------------------------
 // Pre-compilation
@@ -30,7 +17,11 @@
 //---------------------------------------------------------------------------
 //---------------------------------------------------------------------------
 #include "MediaInfo/MediaInfo_Config_MediaInfo.h"
+#include "MediaInfo/MediaInfo_Config.h"
 #include "ZenLib/ZtringListListF.h"
+#if MEDIAINFO_EVENTS
+    #include "ZenLib/FileName.h"
+#endif //MEDIAINFO_EVENTS
 #if MEDIAINFO_IBI
     #include "base64.h"
 #endif //MEDIAINFO_IBI
@@ -57,18 +48,35 @@ MediaInfo_Config_MediaInfo::MediaInfo_Config_MediaInfo()
     FileIsSub=false;
     FileIsDetectingDuration=false;
     FileIsReferenced=false;
+    FileTestContinuousFileNames=true;
     FileKeepInfo=false;
     FileStopAfterFilled=false;
     FileStopSubStreamAfterFilled=false;
     Audio_MergeMonoStreams=false;
     File_Demux_Interleave=false;
     File_ID_OnlyRoot=false;
+    #if MEDIAINFO_ADVANCED
+        File_IgnoreSequenceFileSize=false;
+        File_DefaultFrameRate=0;
+        File_Source_List=false;
+        File_RiskyBitRateEstimation=false;
+        #if MEDIAINFO_DEMUX
+            File_Demux_Unpacketize_StreamLayoutChange_Skip=false;
+        #endif //MEDIAINFO_DEMUX
+    #endif //MEDIAINFO_ADVANCED
+    #if MEDIAINFO_MD5
+        File_Md5=false;
+    #endif //MEDIAINFO_MD5
+    #if defined(MEDIAINFO_REFERENCES_YES)
+        File_CheckSideCarFiles=false;
+    #endif //defined(MEDIAINFO_REFERENCES_YES)
     File_TimeToLive=0;
     File_Buffer_Size_Hint_Pointer=NULL;
     #if MEDIAINFO_NEXTPACKET
         NextPacket=false;
     #endif //MEDIAINFO_NEXTPACKET
     #if MEDIAINFO_FILTER
+        File_Filter_Audio=false;
         File_Filter_HasChanged_=false;
     #endif //MEDIAINFO_FILTER
     #if MEDIAINFO_EVENTS
@@ -80,6 +88,8 @@ MediaInfo_Config_MediaInfo::MediaInfo_Config_MediaInfo()
     #if MEDIAINFO_DEMUX
         Demux_ForceIds=false;
         Demux_PCM_20bitTo16bit=false;
+        Demux_PCM_20bitTo24bit=false;
+        Demux_Avc_Transcode_Iso14496_15_to_Iso14496_10=false;
         Demux_Unpacketize=false;
         Demux_Rate=0;
         Demux_FirstDts=(int64u)-1;
@@ -88,6 +98,7 @@ MediaInfo_Config_MediaInfo::MediaInfo_Config_MediaInfo()
     #endif //MEDIAINFO_DEMUX
     #if MEDIAINFO_IBI
         Ibi_Create=false;
+        Ibi_UseIbiInfoIfAvailable=false;
     #endif //MEDIAINFO_IBI
 
     //Specific
@@ -113,6 +124,9 @@ MediaInfo_Config_MediaInfo::MediaInfo_Config_MediaInfo()
     File_Eia608_DisplayEmptyStream=false;
     File_Eia708_DisplayEmptyStream=false;
     State=0;
+    #if defined(MEDIAINFO_AC3_YES)
+    File_Ac3_IgnoreCrc=false;
+    #endif //defined(MEDIAINFO_AC3_YES)
 
     //Internal to MediaInfo, not thread safe
     File_Names_Pos=0;
@@ -124,10 +138,16 @@ MediaInfo_Config_MediaInfo::MediaInfo_Config_MediaInfo()
     File_Buffer_Repeat_IsSupported=false;
     File_IsGrowing=false;
     File_IsNotGrowingAnymore=false;
+    File_Current_Offset=0;
+    File_Current_Size=(int64u)-1;
     File_Size=(int64u)-1;
     ParseSpeed=MediaInfoLib::Config.ParseSpeed_Get();
     #if MEDIAINFO_DEMUX
         Demux_EventWasSent=false;
+        Demux_Offset_Frame=(int64u)-1;
+        Demux_Offset_DTS=(int64u)-1;
+        Demux_Offset_DTS_FromStream=(int64u)-1;
+        Events_Delayed_CurrentSource=NULL;
         #if MEDIAINFO_SEEK
            Demux_IsSeeking=false;
         #endif //MEDIAINFO_SEEK
@@ -137,6 +157,12 @@ MediaInfo_Config_MediaInfo::MediaInfo_Config_MediaInfo()
 MediaInfo_Config_MediaInfo::~MediaInfo_Config_MediaInfo()
 {
     delete[] File_Buffer; //File_Buffer=NULL;
+
+    #if MEDIAINFO_EVENTS
+        for (events_delayed::iterator Event=Events_Delayed.begin(); Event!=Events_Delayed.end(); ++Event)
+            for (size_t Pos=0; Pos<Event->second.size(); Pos++)
+                delete Event->second[Pos]; //Event->second[Pos]=NULL;
+    #endif //MEDIAINFO_EVENTS
 }
 
 //***************************************************************************
@@ -191,6 +217,15 @@ Ztring MediaInfo_Config_MediaInfo::Option (const String &Option, const String &V
     {
         return File_IsReferenced_Get()?"1":"0";
     }
+    if (Option_Lower==__T("file_testcontinuousfilenames"))
+    {
+        File_TestContinuousFileNames_Set(!(Value==__T("0") || Value.empty()));
+        return __T("");
+    }
+    else if (Option_Lower==__T("file_testcontinuousfilenames_get"))
+    {
+        return File_TestContinuousFileNames_Get()?"1":"0";
+    }
     if (Option_Lower==__T("file_keepinfo"))
     {
         File_KeepInfo_Set(!(Value==__T("0") || Value.empty()));
@@ -245,6 +280,73 @@ Ztring MediaInfo_Config_MediaInfo::Option (const String &Option, const String &V
     {
         return File_ID_OnlyRoot_Get()?"1":"0";
     }
+    else if (Option_Lower==__T("file_ignoresequencefilesize"))
+    {
+        #if MEDIAINFO_MD5
+            File_IgnoreSequenceFileSize_Set(!(Value==__T("0") || Value.empty()));
+            return Ztring();
+        #else //MEDIAINFO_MD5
+            return __T("Disabled due to compilation options");
+        #endif //MEDIAINFO_MD5
+    }
+    else if (Option_Lower==__T("file_defaultframerate"))
+    {
+        #if MEDIAINFO_MD5
+            File_DefaultFrameRate_Set(Ztring(Value).To_float64());
+            return Ztring();
+        #else //MEDIAINFO_MD5
+            return __T("File_DefaultFrameRate is disabled due to compilation options");
+        #endif //MEDIAINFO_MD5
+    }
+    else if (Option_Lower==__T("file_source_list"))
+    {
+        #if MEDIAINFO_MD5
+            File_Source_List_Set(!(Value==__T("0") || Value.empty()));
+            return Ztring();
+        #else //MEDIAINFO_MD5
+            return __T("MD5 is disabled due to compilation options");
+        #endif //MEDIAINFO_MD5
+    }
+    else if (Option_Lower==__T("file_riskybitrateestimation"))
+    {
+        #if MEDIAINFO_ADVANCED
+            File_RiskyBitRateEstimation_Set(!(Value==__T("0") || Value.empty()));
+            return Ztring();
+        #else //MEDIAINFO_ADVANCED
+            return __T("Advanced features are disabled due to compilation options");
+        #endif //MEDIAINFO_ADVANCED
+    }
+    else if (Option_Lower==__T("file_demux_unpacketize_streamlayoutchange_skip"))
+    {
+        #if MEDIAINFO_DEMUX
+            #if MEDIAINFO_ADVANCED
+                File_Demux_Unpacketize_StreamLayoutChange_Skip_Set(!(Value==__T("0") || Value.empty()));
+                return Ztring();
+            #else //MEDIAINFO_ADVANCED
+                return __T("Advanced features disabled due to compilation options");
+            #endif //MEDIAINFO_ADVANCED
+        #else //MEDIAINFO_ADVANCED
+            return __T("Advanced features disabled due to compilation options");
+        #endif //MEDIAINFO_ADVANCED
+    }
+    else if (Option_Lower==__T("file_md5"))
+    {
+        #if MEDIAINFO_MD5
+            File_Md5_Set(!(Value==__T("0") || Value.empty()));
+            return Ztring();
+        #else //MEDIAINFO_MD5
+            return __T("MD5 is disabled due to compilation options");
+        #endif //MEDIAINFO_MD5
+    }
+    else if (Option_Lower==__T("file_checksidecarfiles"))
+    {
+        #if defined(MEDIAINFO_REFERENCES_YES)
+            File_CheckSideCarFiles_Set(!(Value==__T("0") || Value.empty()));
+            return Ztring();
+        #else //defined(MEDIAINFO_REFERENCES_YES)
+            return __T("Disabled due to compilation options");
+        #endif //defined(MEDIAINFO_REFERENCES_YES)
+    }
     else if (Option_Lower==__T("file_filename"))
     {
         File_FileName_Set(Value);
@@ -311,8 +413,12 @@ Ztring MediaInfo_Config_MediaInfo::Option (const String &Option, const String &V
     else if (Option_Lower==__T("file_filter"))
     {
         #if MEDIAINFO_FILTER
-            File_Filter_Set(Ztring(Value).To_int64u());
-            return __T("");
+            Ztring ValueLowerCase=Ztring(Value).MakeLowerCase();
+            if (ValueLowerCase==__T("audio"))
+                File_Filter_Audio_Set(true);
+            else
+                File_Filter_Set(ValueLowerCase.To_int64u());
+            return Ztring();
         #else //MEDIAINFO_FILTER
             return __T("Filter manager is disabled due to compilation options");
         #endif //MEDIAINFO_FILTER
@@ -368,6 +474,30 @@ Ztring MediaInfo_Config_MediaInfo::Option (const String &Option, const String &V
             return __T("Demux manager is disabled due to compilation options");
         #endif //MEDIAINFO_DEMUX
     }
+    else if (Option_Lower==__T("file_demux_pcm_20bitto24bit"))
+    {
+        #if MEDIAINFO_DEMUX
+            if (Value.empty())
+                Demux_PCM_20bitTo24bit_Set(false);
+            else
+                Demux_PCM_20bitTo24bit_Set(true);
+            return Ztring();
+        #else //MEDIAINFO_DEMUX
+            return __T("Demux manager is disabled due to compilation options");
+        #endif //MEDIAINFO_DEMUX
+    }
+    else if (Option_Lower==__T("file_demux_avc_transcode_iso14496_15_to_iso14496_10"))
+    {
+        #if MEDIAINFO_DEMUX
+            if (Value.empty())
+                Demux_Avc_Transcode_Iso14496_15_to_Iso14496_10_Set(false);
+            else
+                Demux_Avc_Transcode_Iso14496_15_to_Iso14496_10_Set(true);
+            return Ztring();
+        #else //MEDIAINFO_DEMUX
+            return __T("Demux manager is disabled due to compilation options");
+        #endif //MEDIAINFO_DEMUX
+    }
     else if (Option_Lower==__T("file_demux_unpacketize"))
     {
         #if MEDIAINFO_DEMUX
@@ -465,6 +595,18 @@ Ztring MediaInfo_Config_MediaInfo::Option (const String &Option, const String &V
             return __T("IBI support is disabled due to compilation options");
         #endif //MEDIAINFO_IBI
     }
+    else if (Option_Lower==__T("file_ibi_useibiinfoifavailable"))
+    {
+        #if MEDIAINFO_IBI
+            if (Value.empty())
+                Ibi_UseIbiInfoIfAvailable_Set(false);
+            else
+                Ibi_UseIbiInfoIfAvailable_Set(true);
+            return Ztring();
+        #else //MEDIAINFO_IBI
+            return __T("IBI support is disabled due to compilation options");
+        #endif //MEDIAINFO_IBI
+    }
     else if (Option_Lower==__T("file_nextpacket"))
     {
         #if MEDIAINFO_NEXTPACKET
@@ -687,6 +829,23 @@ Ztring MediaInfo_Config_MediaInfo::Option (const String &Option, const String &V
     {
         return File_Eia608_DisplayEmptyStream_Get()?"1":"0";
     }
+    else if (Option_Lower==__T("file_ac3_ignorecrc"))
+    {
+        #if defined(MEDIAINFO_AC3_YES)
+            File_Ac3_IgnoreCrc_Set(!(Value==__T("0") || Value.empty()));
+            return __T("");
+        #else //defined(MEDIAINFO_AC3_YES)
+            return __T("AC-3 support is disabled due to compilation options");
+        #endif //defined(MEDIAINFO_AC3_YES)
+    }
+    else if (Option_Lower==__T("file_ac3_ignorecrc_get"))
+    {
+        #if defined(MEDIAINFO_AC3_YES)
+            return File_Ac3_IgnoreCrc_Get()?"1":"0";
+        #else //defined(MEDIAINFO_AC3_YES)
+            return __T("AC-3 support is disabled due to compilation options");
+        #endif //defined(MEDIAINFO_AC3_YES)
+    }
     else if (Option_Lower==__T("file_event_callbackfunction"))
     {
         #if MEDIAINFO_EVENTS
@@ -785,7 +944,24 @@ bool MediaInfo_Config_MediaInfo::File_KeepInfo_Get ()
 }
 
 //***************************************************************************
-// File Keep Info
+// File test continuous file names
+//***************************************************************************
+
+//---------------------------------------------------------------------------
+void MediaInfo_Config_MediaInfo::File_TestContinuousFileNames_Set (bool NewValue)
+{
+    CriticalSectionLocker CSL(CS);
+    FileTestContinuousFileNames=NewValue;
+}
+
+bool MediaInfo_Config_MediaInfo::File_TestContinuousFileNames_Get ()
+{
+    CriticalSectionLocker CSL(CS);
+    return FileTestContinuousFileNames;
+}
+
+//***************************************************************************
+// Stop after filled
 //***************************************************************************
 
 //---------------------------------------------------------------------------
@@ -853,6 +1029,116 @@ bool MediaInfo_Config_MediaInfo::File_ID_OnlyRoot_Get ()
     return File_ID_OnlyRoot;
 }
 
+//---------------------------------------------------------------------------
+#if MEDIAINFO_MD5
+void MediaInfo_Config_MediaInfo::File_Md5_Set (bool NewValue)
+{
+    CriticalSectionLocker CSL(CS);
+    File_Md5=NewValue;
+}
+
+bool MediaInfo_Config_MediaInfo::File_Md5_Get ()
+{
+    CriticalSectionLocker CSL(CS);
+    return File_Md5;
+}
+#endif //MEDIAINFO_MD5
+
+//---------------------------------------------------------------------------
+#if defined(MEDIAINFO_REFERENCES_YES)
+void MediaInfo_Config_MediaInfo::File_CheckSideCarFiles_Set (bool NewValue)
+{
+    CriticalSectionLocker CSL(CS);
+    File_CheckSideCarFiles=NewValue;
+}
+
+bool MediaInfo_Config_MediaInfo::File_CheckSideCarFiles_Get ()
+{
+    CriticalSectionLocker CSL(CS);
+    return File_CheckSideCarFiles;
+}
+#endif //defined(MEDIAINFO_REFERENCES_YES)
+
+//---------------------------------------------------------------------------
+#if MEDIAINFO_ADVANCED
+void MediaInfo_Config_MediaInfo::File_IgnoreSequenceFileSize_Set (bool NewValue)
+{
+    CriticalSectionLocker CSL(CS);
+    File_IgnoreSequenceFileSize=NewValue;
+}
+
+bool MediaInfo_Config_MediaInfo::File_IgnoreSequenceFileSize_Get ()
+{
+    CriticalSectionLocker CSL(CS);
+    return File_IgnoreSequenceFileSize;
+}
+#endif //MEDIAINFO_ADVANCED
+
+//---------------------------------------------------------------------------
+#if MEDIAINFO_ADVANCED
+void MediaInfo_Config_MediaInfo::File_DefaultFrameRate_Set (float64 NewValue)
+{
+    CriticalSectionLocker CSL(CS);
+    File_DefaultFrameRate=NewValue;
+    #if MEDIAINFO_DEMUX
+        Demux_Rate=File_DefaultFrameRate;
+    #endif //MEDIAINFO_DEMUX
+}
+
+float64 MediaInfo_Config_MediaInfo::File_DefaultFrameRate_Get ()
+{
+    CriticalSectionLocker CSL(CS);
+    return File_DefaultFrameRate;
+}
+#endif //MEDIAINFO_ADVANCED
+
+//---------------------------------------------------------------------------
+#if MEDIAINFO_ADVANCED
+void MediaInfo_Config_MediaInfo::File_Source_List_Set (bool NewValue)
+{
+    CriticalSectionLocker CSL(CS);
+    File_Source_List=NewValue;
+}
+
+bool MediaInfo_Config_MediaInfo::File_Source_List_Get ()
+{
+    CriticalSectionLocker CSL(CS);
+    return File_Source_List;
+}
+#endif //MEDIAINFO_ADVANCED
+
+//---------------------------------------------------------------------------
+#if MEDIAINFO_ADVANCED
+void MediaInfo_Config_MediaInfo::File_RiskyBitRateEstimation_Set (bool NewValue)
+{
+    CriticalSectionLocker CSL(CS);
+    File_RiskyBitRateEstimation=NewValue;
+}
+
+bool MediaInfo_Config_MediaInfo::File_RiskyBitRateEstimation_Get ()
+{
+    CriticalSectionLocker CSL(CS);
+    return File_RiskyBitRateEstimation;
+}
+#endif //MEDIAINFO_ADVANCED
+
+//---------------------------------------------------------------------------
+#if MEDIAINFO_DEMUX
+#if MEDIAINFO_ADVANCED
+void MediaInfo_Config_MediaInfo::File_Demux_Unpacketize_StreamLayoutChange_Skip_Set (bool NewValue)
+{
+    CriticalSectionLocker CSL(CS);
+    File_Demux_Unpacketize_StreamLayoutChange_Skip=NewValue;
+}
+
+bool MediaInfo_Config_MediaInfo::File_Demux_Unpacketize_StreamLayoutChange_Skip_Get ()
+{
+    CriticalSectionLocker CSL(CS);
+    return File_Demux_Unpacketize_StreamLayoutChange_Skip;
+}
+#endif //MEDIAINFO_ADVANCED
+#endif //MEDIAINFO_DEMUX
+
 //***************************************************************************
 // File name from somewhere else
 //***************************************************************************
@@ -952,7 +1238,11 @@ Ztring MediaInfo_Config_MediaInfo::File_ForceParser_Get ()
 }
 
 //***************************************************************************
-// File_Buffer_Size_Hint_Pointer
+/*  Copyright (c) MediaArea.net SARL. All Rights Reserved.
+ *
+ *  Use of this source code is governed by a BSD-style license that can
+ *  be found in the License.html file in the root of the source tree.
+ */
 //***************************************************************************
 
 //---------------------------------------------------------------------------
@@ -1000,6 +1290,18 @@ bool MediaInfo_Config_MediaInfo::File_Filter_Get ()
     return Exist;
 }
 
+void MediaInfo_Config_MediaInfo::File_Filter_Audio_Set (bool NewValue)
+{
+    CriticalSectionLocker CSL(CS);
+    File_Filter_Audio=NewValue;
+}
+
+bool MediaInfo_Config_MediaInfo::File_Filter_Audio_Get ()
+{
+    CriticalSectionLocker CSL(CS);
+    return File_Filter_Audio;
+}
+
 bool MediaInfo_Config_MediaInfo::File_Filter_HasChanged ()
 {
     CriticalSectionLocker CSL(CS);
@@ -1133,6 +1435,32 @@ bool MediaInfo_Config_MediaInfo::Demux_PCM_20bitTo16bit_Get ()
     return Demux_PCM_20bitTo16bit;
 }
 
+//---------------------------------------------------------------------------
+void MediaInfo_Config_MediaInfo::Demux_PCM_20bitTo24bit_Set (bool NewValue)
+{
+    CriticalSectionLocker CSL(CS);
+    Demux_PCM_20bitTo24bit=NewValue;
+}
+
+bool MediaInfo_Config_MediaInfo::Demux_PCM_20bitTo24bit_Get ()
+{
+    CriticalSectionLocker CSL(CS);
+    return Demux_PCM_20bitTo24bit;
+}
+
+//---------------------------------------------------------------------------
+void MediaInfo_Config_MediaInfo::Demux_Avc_Transcode_Iso14496_15_to_Iso14496_10_Set (bool NewValue)
+{
+    CriticalSectionLocker CSL(CS);
+    Demux_Avc_Transcode_Iso14496_15_to_Iso14496_10=NewValue;
+}
+
+bool MediaInfo_Config_MediaInfo::Demux_Avc_Transcode_Iso14496_15_to_Iso14496_10_Get ()
+{
+    CriticalSectionLocker CSL(CS);
+    return Demux_Avc_Transcode_Iso14496_15_to_Iso14496_10;
+}
+
 //---------------------------------------------------------------------------
 void MediaInfo_Config_MediaInfo::Demux_Unpacketize_Set (bool NewValue)
 {
@@ -1226,6 +1554,7 @@ string MediaInfo_Config_MediaInfo::Ibi_Get ()
     CriticalSectionLocker CSL(CS);
     return Ibi;
 }
+
 //---------------------------------------------------------------------------
 void MediaInfo_Config_MediaInfo::Ibi_Create_Set (bool NewValue)
 {
@@ -1238,6 +1567,19 @@ bool MediaInfo_Config_MediaInfo::Ibi_Create_Get ()
     CriticalSectionLocker CSL(CS);
     return Ibi_Create;
 }
+
+//---------------------------------------------------------------------------
+void MediaInfo_Config_MediaInfo::Ibi_UseIbiInfoIfAvailable_Set (bool NewValue)
+{
+    CriticalSectionLocker CSL(CS);
+    Ibi_UseIbiInfoIfAvailable=NewValue;
+}
+
+bool MediaInfo_Config_MediaInfo::Ibi_UseIbiInfoIfAvailable_Get ()
+{
+    CriticalSectionLocker CSL(CS);
+    return Ibi_UseIbiInfoIfAvailable;
+}
 #endif //MEDIAINFO_IBI
 
 //***************************************************************************
@@ -1383,35 +1725,83 @@ Ztring MediaInfo_Config_MediaInfo::Event_CallBackFunction_Get ()
 
 //---------------------------------------------------------------------------
 #if MEDIAINFO_EVENTS
-void MediaInfo_Config_MediaInfo::Event_Send (const int8u* Data_Content, size_t Data_Size)
+void MediaInfo_Config_MediaInfo::Event_Send (File__Analyze* Source, const int8u* Data_Content, size_t Data_Size, const Ztring &File_Name)
 {
     CriticalSectionLocker CSL(CS);
 
-    if (Event_CallBackFunction)
-        Event_CallBackFunction ((unsigned char*)Data_Content, Data_Size, Event_UserHandler);
-}
-#endif //MEDIAINFO_EVENTS
+    if (Source==NULL)
+    {
+        if (Demux_Offset_Frame!=(int64u)-1)
+        {
+            MediaInfo_Event_Generic* Temp=(MediaInfo_Event_Generic*)Data_Content;
+            if (Temp->FrameNumber!=(int64u)-1)
+                Temp->FrameNumber+=Demux_Offset_Frame;
+            if (Temp->FrameNumber_PresentationOrder!=(int64u)-1)
+                Temp->FrameNumber_PresentationOrder+=Demux_Offset_Frame;
+        }
+        if (Demux_Offset_DTS!=(int64u)-1) // && (Demux_Offset_DTS_FromStream==(int64u)-1 || Demux_Offset_DTS!=Demux_Offset_DTS_FromStream))
+        {
+            MediaInfo_Event_Generic* Temp=(MediaInfo_Event_Generic*)Data_Content;
+            if (Temp->DTS!=(int64u)-1)
+            {
+                Temp->DTS+=Demux_Offset_DTS;
+                if (Demux_Offset_DTS_FromStream!=(int64u)-1)
+                    Temp->DTS-=Demux_Offset_DTS_FromStream;
+            }
+            if (Temp->PTS!=(int64u)-1)
+            {
+                Temp->PTS+=Demux_Offset_DTS;
+                if (Demux_Offset_DTS_FromStream!=(int64u)-1)
+                    Temp->PTS-=Demux_Offset_DTS_FromStream;
+            }
+        }
+    }
+    
+    if (Source)
+    {
+        event_delayed* Event=new event_delayed(Data_Content, Data_Size, File_Name);
+        Events_Delayed[Source].push_back(Event);
 
-//---------------------------------------------------------------------------
-#if MEDIAINFO_EVENTS
-void MediaInfo_Config_MediaInfo::Event_Send (const int8u* Data_Content, size_t Data_Size, const Ztring &File_Name)
-{
-    CriticalSectionLocker CSL(CS);
-
-    if (Event_CallBackFunction)
+        // Copying buffers
+        int32u* EventCode=(int32u*)Data_Content;
+        if (((*EventCode)&0x00FFFFFF)==((MediaInfo_Event_Global_Demux<<8)|4) && Data_Size==sizeof(MediaInfo_Event_Global_Demux_4))
+        {
+            MediaInfo_Event_Global_Demux_4* Old=(MediaInfo_Event_Global_Demux_4*)Data_Content;
+            MediaInfo_Event_Global_Demux_4* New=(MediaInfo_Event_Global_Demux_4*)Event->Data_Content;
+            if (New->Content_Size)
+            {
+                int8u* Content=new int8u[New->Content_Size];
+                std::memcpy(Content, Old->Content, New->Content_Size*sizeof(int8u));
+                New->Content=Content;
+            }
+            if (New->Offsets_Size)
+            {
+                int64u* Offsets_Stream=new int64u[New->Offsets_Size];
+                std::memcpy(Offsets_Stream, Old->Offsets_Stream, New->Offsets_Size*sizeof(int64u));
+                New->Offsets_Stream=Offsets_Stream;
+                int64u* Offsets_Content=new int64u[New->Offsets_Size];
+                std::memcpy(Offsets_Content, Old->Offsets_Content, New->Offsets_Size*sizeof(int64u));
+                New->Offsets_Content=Offsets_Content;
+            }
+            if (New->OriginalContent_Size)
+            {
+                int8u* OriginalContent=new int8u[New->OriginalContent_Size];
+                std::memcpy(OriginalContent, Old->OriginalContent, New->OriginalContent_Size*sizeof(int8u));
+                New->OriginalContent=OriginalContent;
+            }
+        }
+    }
+    else if (Event_CallBackFunction)
         Event_CallBackFunction ((unsigned char*)Data_Content, Data_Size, Event_UserHandler);
-    else
+    else if (!File_Name.empty())
     {
         MediaInfo_Event_Generic* Event_Generic=(MediaInfo_Event_Generic*)Data_Content;
-        if ((Event_Generic->EventCode&0x00FFFF03)==((MediaInfo_Event_Global_Demux<<8)|0x03)) //Demux version 3
+        if ((Event_Generic->EventCode&0x00FFFFFF)==((MediaInfo_Event_Global_Demux<<8)|0x04)) //Demux version 4
         {
             if (!MediaInfoLib::Config.Demux_Get())
                 return;
 
-            if (File_Name.empty())
-                return;
-
-            MediaInfo_Event_Global_Demux_0* Event=(MediaInfo_Event_Global_Demux_0*)Data_Content;
+            MediaInfo_Event_Global_Demux_4* Event=(MediaInfo_Event_Global_Demux_4*)Data_Content;
 
             Ztring File_Name_Final(File_Name);
             if (Event->StreamIDs_Size==0)
@@ -1444,6 +1834,99 @@ void MediaInfo_Config_MediaInfo::Event_Send (const int8u* Data_Content, size_t D
         }
     }
 }
+
+void MediaInfo_Config_MediaInfo::Event_Accepted (File__Analyze* Source)
+{
+    #if MEDIAINFO_DEMUX && MEDIAINFO_NEXTPACKET
+        if (Demux_EventWasSent && NextPacket_Get())
+        {
+            Events_Delayed_CurrentSource=Source;
+            return;
+        }
+    #endif //MEDIAINFO_DEMUX && MEDIAINFO_NEXTPACKET
+
+    for (events_delayed::iterator Event=Events_Delayed.begin(); Event!=Events_Delayed.end(); ++Event)
+        if (Event->first==Source)
+        {
+            for (size_t Pos=0; Pos<Event->second.size(); Pos++)
+                if (Event->second[Pos])
+                {
+                    Event_Send(NULL, Event->second[Pos]->Data_Content, Event->second[Pos]->Data_Size, Event->second[Pos]->File_Name);
+
+                    int32u EventCode=*((int32u*)Event->second[Pos]->Data_Content);
+                    bool IsDemux=(EventCode&0x00FFFF00)==(MediaInfo_Event_Global_Demux<<8);
+
+                    if (IsDemux)
+                    {
+                        MediaInfo_Event_Global_Demux_4* Old=(MediaInfo_Event_Global_Demux_4*)Event->second[Pos]->Data_Content;
+                        delete[] Old->Content; Old->Content=NULL;
+                        if (Old->Offsets_Size)
+                        {
+                            delete[] Old->Offsets_Content; Old->Offsets_Content=NULL;
+                        }
+                        if (Old->Offsets_Size)
+                        {
+                            delete[] Old->OriginalContent; Old->OriginalContent=NULL;
+                        }
+                    }
+
+                    delete Event->second[Pos]; Event->second[Pos]=NULL;
+
+                    #if MEDIAINFO_DEMUX && MEDIAINFO_NEXTPACKET
+                        if (IsDemux && NextPacket_Get())
+                        {
+                            Demux_EventWasSent=true;
+                            Event->second.erase(Event->second.begin(), Event->second.begin()+Pos);
+                            Events_Delayed_CurrentSource=Source;
+                            return;
+                        }
+                    #endif //MEDIAINFO_DEMUX && MEDIAINFO_NEXTPACKET
+                }
+
+            Events_Delayed.erase(Event->first);
+            return;
+        }
+}
+
+//---------------------------------------------------------------------------
+void MediaInfo_Config_MediaInfo::Event_SubFile_Start(const Ztring &FileName_Absolute)
+{
+    Ztring FileName_Relative;
+    if (File_Names_RootDirectory.empty())
+    {
+        FileName FN(FileName_Absolute);
+        FileName_Relative=FN.Name_Get();
+        if (!FN.Extension_Get().empty())
+        {
+            FileName_Relative+=__T('.');
+            FileName_Relative+=FN.Extension_Get();
+        }
+    }
+    else
+    {
+        Ztring Root=File_Names_RootDirectory+PathSeparator;
+        FileName_Relative=FileName_Absolute;
+        if (FileName_Relative.find(Root)==0)
+            FileName_Relative.erase(0, Root.size());
+    }
+
+    struct MediaInfo_Event_General_SubFile_Start_0 Event;
+    memset(&Event, 0xFF, sizeof(struct MediaInfo_Event_Generic));
+    Event.EventCode=MediaInfo_EventCode_Create(0, MediaInfo_Event_General_SubFile_Start, 0);
+    Event.EventSize=sizeof(struct MediaInfo_Event_General_SubFile_Start_0);
+    Event.StreamIDs_Size=0;
+
+    std::string FileName_Relative_Ansi=FileName_Relative.To_UTF8();
+    std::wstring FileName_Relative_Unicode=FileName_Relative.To_Unicode();
+    std::string FileName_Absolute_Ansi=FileName_Absolute.To_UTF8();
+    std::wstring FileName_Absolute_Unicode=FileName_Absolute.To_Unicode();
+    Event.FileName_Relative=FileName_Relative_Ansi.c_str();
+    Event.FileName_Relative_Unicode=FileName_Relative_Unicode.c_str();
+    Event.FileName_Absolute=FileName_Absolute_Ansi.c_str();
+    Event.FileName_Absolute_Unicode=FileName_Absolute_Unicode.c_str();
+
+    Event_Send(NULL, (const int8u*)&Event, Event.EventSize);
+}
 #endif //MEDIAINFO_EVENTS
 
 //***************************************************************************
@@ -1680,6 +2163,22 @@ bool MediaInfo_Config_MediaInfo::File_Eia708_DisplayEmptyStream_Get ()
     return Temp;
 }
 
+//---------------------------------------------------------------------------
+#if defined(MEDIAINFO_AC3_YES)
+void MediaInfo_Config_MediaInfo::File_Ac3_IgnoreCrc_Set (bool NewValue)
+{
+    CriticalSectionLocker CSL(CS);
+    File_Ac3_IgnoreCrc=NewValue;
+}
+
+bool MediaInfo_Config_MediaInfo::File_Ac3_IgnoreCrc_Get ()
+{
+    CriticalSectionLocker CSL(CS);
+    bool Temp=File_Ac3_IgnoreCrc;
+    return Temp;
+}
+#endif //defined(MEDIAINFO_AC3_YES)
+
 //***************************************************************************
 // Analysis internal
 //***************************************************************************
