@@ -1,21 +1,8 @@
-// MediaInfo_Events - 
-// Copyright (C) 2010-2011 MediaArea.net SARL, Info@MediaArea.net
-//
-// This library is free software: you can redistribute it and/or modify it
-// under the terms of the GNU Lesser General Public License as published by
-// the Free Software Foundation, either version 3 of the License, or
-// any later version.
-//
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-// GNU Lesser General Public License for more details.
-//
-// You should have received a copy of the GNU Lesser General Public License
-// along with this library. If not, see <http://www.gnu.org/licenses/>.
-//
-//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
-//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
+/*  Copyright (c) MediaArea.net SARL. All Rights Reserved.
+ *
+ *  Use of this source code is governed by a BSD-style license that can
+ *  be found in the License.html file in the root of the source tree.
+ */
 
 /*Structures for MediaInfo events */
 
@@ -177,10 +164,12 @@
     #undef  MAXTYPE_INT
     #define MAXTYPE_INT 64
     typedef unsigned long long  MediaInfo_int64u;
+    typedef   signed long long  MediaInfo_int64s;
 #elif defined(__WIN32__) || defined(_WIN32)
     #undef  MAXTYPE_INT
     #define MAXTYPE_INT 64
     typedef unsigned __int64    MediaInfo_int64u;
+    typedef   signed __int64    MediaInfo_int64s;
 #else
     #pragma message This machine has no 64-bit integer type?
 #endif
@@ -220,10 +209,61 @@ extern "C"
 /***************************************************************************/
 
 /*-------------------------------------------------------------------------*/
-/* Generic                                                                 */
-struct MediaInfo_Event_Generic
+/* Time code                                                               */
+typedef struct MediaInfo_time_code
 {
-    MediaInfo_int32u        EventCode;
+    MediaInfo_int8u     Hours;
+    MediaInfo_int8u     Minutes;
+    MediaInfo_int8u     Seconds;
+    MediaInfo_int8u     Frames;
+    MediaInfo_int8u     FramesPerSecond;
+    MediaInfo_int8u     DropFrame; //0= No, 1=Yes
+    MediaInfo_int8u     Reserved[2];
+} MediaInfo_time_code;
+
+/*-------------------------------------------------------------------------*/
+/* Generic                                                                 */
+#define MEDIAINFO_EVENT_GENERIC \
+    MediaInfo_int32u        EventCode; \
+    MediaInfo_int32u        ReservedI32; \
+    size_t                  EventSize; \
+    size_t                  StreamIDs_Size; \
+    MediaInfo_int64u        StreamIDs[16]; \
+    MediaInfo_int8u         StreamIDs_Width[16]; \
+    MediaInfo_int8u         ParserIDs[16]; \
+    MediaInfo_int64u        StreamOffset; \
+    MediaInfo_int64u        FrameNumber; \
+    MediaInfo_int64u        PCR; \
+    MediaInfo_int64u        PTS; \
+    MediaInfo_int64u        DTS; \
+    MediaInfo_int64u        DUR; \
+    MediaInfo_int64u        FrameNumber_PresentationOrder; \
+    MediaInfo_int64u        ReservedI64[1]; \
+    MediaInfo_time_code     TimeCode_Container; \
+    MediaInfo_time_code     TimeCode_SDTI; \
+    MediaInfo_time_code     TimeCode_RawStream; \
+    MediaInfo_time_code     ReservedT[5]; \
+
+typedef struct MediaInfo_Event_Generic
+{
+    MEDIAINFO_EVENT_GENERIC
+} MediaInfo_Event_Generic;
+
+/*-------------------------------------------------------------------------*/
+/* MediaInfo_Event_Log_0                                                   */
+#define MediaInfo_Event_Log 0x0F00
+struct MediaInfo_Event_Log_0
+{
+    MEDIAINFO_EVENT_GENERIC
+    MediaInfo_int8u         Type;
+    MediaInfo_int8u         Severity;
+    MediaInfo_int8u         Reserved2;
+    MediaInfo_int8u         Reserved3;
+    MediaInfo_int32u        MessageCode;
+    MediaInfo_int32u        Reserved4;
+    const wchar_t*          MessageString;
+    const wchar_t*          MessageStringU;
+    const char*             MessageStringA;
 };
 
 /*-------------------------------------------------------------------------*/
@@ -236,71 +276,31 @@ enum MediaInfo_Event_Global_Demux_0_contenttype
     MediaInfo_Event_Global_Demux_0_ContentType_Header,
     MediaInfo_Event_Global_Demux_0_ContentType_Synchro
 };
-struct MediaInfo_Event_Global_Demux_0
+
+struct MediaInfo_Event_Global_Demux_4
 {
-    MediaInfo_int32u        EventCode;
-    MediaInfo_int64u        Stream_Offset;
-    MediaInfo_int64u        PCR;
-    MediaInfo_int64u        PTS;
-    MediaInfo_int64u        DTS;
-    size_t                  StreamIDs_Size;
-    MediaInfo_int64u*       StreamIDs;
-    MediaInfo_int8u*        StreamIDs_Width;
-    MediaInfo_int8u*        ParserIDs;
+    MEDIAINFO_EVENT_GENERIC
     MediaInfo_int8u         Content_Type; /*MediaInfo_Event_Global_Demux_0_contenttype*/
     size_t                  Content_Size;
     const MediaInfo_int8u*  Content;
-};
-struct MediaInfo_Event_Global_Demux_1
-{
-    MediaInfo_int32u        EventCode;
-    MediaInfo_int64u        Stream_Offset;
-    MediaInfo_int64u        PCR;
-    MediaInfo_int64u        PTS;
-    MediaInfo_int64u        DTS;
-    size_t                  StreamIDs_Size;
-    MediaInfo_int64u*       StreamIDs;
-    MediaInfo_int8u*        StreamIDs_Width;
-    MediaInfo_int8u*        ParserIDs;
-    MediaInfo_int8u         Content_Type; /*MediaInfo_Event_Global_Demux_0_contenttype*/
-    size_t                  Content_Size;
-    const MediaInfo_int8u*  Content;
-    MediaInfo_int64u        DUR;
-};
-struct MediaInfo_Event_Global_Demux_2
-{
-    MediaInfo_int32u        EventCode;
-    MediaInfo_int64u        Stream_Offset;
-    MediaInfo_int64u        PCR;
-    MediaInfo_int64u        PTS;
-    MediaInfo_int64u        DTS;
-    size_t                  StreamIDs_Size;
-    MediaInfo_int64u*       StreamIDs;
-    MediaInfo_int8u*        StreamIDs_Width;
-    MediaInfo_int8u*        ParserIDs;
-    MediaInfo_int8u         Content_Type; /*MediaInfo_Event_Global_Demux_0_contenttype*/
-    size_t                  Content_Size;
-    const MediaInfo_int8u*  Content;
-    MediaInfo_int64u        DUR;
     MediaInfo_int64u        Flags; /*bit0=random_access*/
+    size_t                  Offsets_Size;
+    const MediaInfo_int64u* Offsets_Stream; /* From the begin of the stream */
+    const MediaInfo_int64u* Offsets_Content; /* From the begin of the demuxed content */
+    size_t                  OriginalContent_Size; /* In case of decoded content inside MediaInfo, OriginalContent contain the not-decoded stream */
+    const MediaInfo_int8u*  OriginalContent; /* In case of decoded content inside MediaInfo, OriginalContent contain the not-decoded stream */
 };
-struct MediaInfo_Event_Global_Demux_3
+
+/*-------------------------------------------------------------------------*/
+/* MediaInfo_Event_Video_SliceInfo_0                                       */
+#define MediaInfo_Event_Video_SliceInfo 0x7801
+struct MediaInfo_Event_Video_SliceInfo_0
 {
-    MediaInfo_int32u        EventCode;
-    MediaInfo_int64u        Stream_Offset;
-    MediaInfo_int64u        PCR;
-    MediaInfo_int64u        PTS;
-    MediaInfo_int64u        DTS;
-    size_t                  StreamIDs_Size;
-    MediaInfo_int64u*       StreamIDs;
-    MediaInfo_int8u*        StreamIDs_Width;
-    MediaInfo_int8u*        ParserIDs;
-    MediaInfo_int8u         Content_Type; /*MediaInfo_Event_Global_Demux_0_contenttype*/
-    size_t                  Content_Size;
-    const MediaInfo_int8u*  Content;
-    MediaInfo_int64u        DUR;
-    MediaInfo_int64u        Flags; /*bit0=random_access*/
-    MediaInfo_int64u        FrameNumber;
+    MEDIAINFO_EVENT_GENERIC
+    MediaInfo_int64u        FieldPosition;
+    MediaInfo_int64u        SlicePosition;
+    MediaInfo_int8u         SliceType;
+    MediaInfo_int64u        Flags;
 };
 
 /***************************************************************************/
@@ -308,14 +308,19 @@ struct MediaInfo_Event_Global_Demux_3
 /***************************************************************************/
 
 #define MediaInfo_Parser_None           0x00
+#define MediaInfo_Parser_General        0x00
+#define MediaInfo_Parser_Global         0x00
+#define MediaInfo_Parser_Video          0x01
 
 /*-------------------------------------------------------------------------*/
 /* Start                                                                   */
 #define MediaInfo_Event_General_Start 0x7001
 struct MediaInfo_Event_General_Start_0
 {
-    MediaInfo_int32u        EventCode;
+    MEDIAINFO_EVENT_GENERIC
     MediaInfo_int64u        Stream_Size;
+    const char*             FileName;
+    const wchar_t*          FileName_Unicode;
 };
 
 /*-------------------------------------------------------------------------*/
@@ -323,8 +328,11 @@ struct MediaInfo_Event_General_Start_0
 #define MediaInfo_Event_General_End 0x7002
 struct MediaInfo_Event_General_End_0
 {
-    MediaInfo_int32u        EventCode;
+    MEDIAINFO_EVENT_GENERIC
     MediaInfo_int64u        Stream_Bytes_Analyzed;
+    MediaInfo_int64u        Stream_Size;
+    MediaInfo_int64u        Stream_Bytes_Padding;
+    MediaInfo_int64u        Stream_Bytes_Junk;
 };
 
 /*-------------------------------------------------------------------------*/
@@ -332,7 +340,7 @@ struct MediaInfo_Event_General_End_0
 #define MediaInfo_Event_General_Parser_Selected 0x7003
 struct MediaInfo_Event_General_Parser_Selected_0
 {
-    MediaInfo_int32u        EventCode;
+    MEDIAINFO_EVENT_GENERIC
     char                    Name[16];
 };
 
@@ -341,8 +349,7 @@ struct MediaInfo_Event_General_Parser_Selected_0
 #define MediaInfo_Event_General_Move_Request 0x7004
 struct MediaInfo_Event_General_Move_Request_0
 {
-    MediaInfo_int32u        EventCode;
-    MediaInfo_int64u        Stream_Offset;
+    MEDIAINFO_EVENT_GENERIC
 };
 
 /*-------------------------------------------------------------------------*/
@@ -350,10 +357,35 @@ struct MediaInfo_Event_General_Move_Request_0
 #define MediaInfo_Event_General_Move_Done 0x7005
 struct MediaInfo_Event_General_Move_Done_0
 {
-    MediaInfo_int32u        EventCode;
-    MediaInfo_int64u        Stream_Offset;
+    MEDIAINFO_EVENT_GENERIC
 };
 
+/*-------------------------------------------------------------------------*/
+/* SubFile_Start                                                           */
+#define MediaInfo_Event_General_SubFile_Start 0x7006
+struct MediaInfo_Event_General_SubFile_Start_0
+{
+    MEDIAINFO_EVENT_GENERIC
+    const char*             FileName_Relative;
+    const wchar_t*          FileName_Relative_Unicode;
+    const char*             FileName_Absolute;
+    const wchar_t*          FileName_Absolute_Unicode;
+};
+
+/*-------------------------------------------------------------------------*/
+/* SubFile_End                                                             */
+#define MediaInfo_Event_General_SubFile_End 0x7007
+struct MediaInfo_Event_General_SubFile_End_0
+{
+    MEDIAINFO_EVENT_GENERIC
+};
+
+/***************************************************************************/
+/* MPEG-TS / BDAV / TSP                                                    */
+/***************************************************************************/
+
+#define MediaInfo_Parser_SideCar        0x72
+
 /***************************************************************************/
 /* MPEG-TS / BDAV / TSP                                                    */
 /***************************************************************************/
@@ -416,6 +448,7 @@ struct MediaInfo_Event_DvDif_Analysis_Frame_0
 /***************************************************************************/
 
 #define MediaInfo_Parser_Mpeg4          0x09
+#define MediaInfo_Parser_Mpeg4_Desc     0x71
 
 /***************************************************************************/
 /* MXF                                                                     */
@@ -447,6 +480,54 @@ struct MediaInfo_Event_DvDif_Analysis_Frame_0
 
 #define MediaInfo_Parser_Lxf            0x0E
 
+/***************************************************************************/
+/* HLS                                                                     */
+/***************************************************************************/
+
+#define MediaInfo_Parser_Hls            0x60
+
+/***************************************************************************/
+/* HLS Index                                                               */
+/***************************************************************************/
+
+#define MediaInfo_Parser_HlsIndex       0x61
+
+/***************************************************************************/
+/* Internet Streaming Media                                                */
+/***************************************************************************/
+
+#define MediaInfo_Parser_Ism            0x62
+
+/***************************************************************************/
+/* DASH MPD                                                                */
+/***************************************************************************/
+
+#define MediaInfo_Parser_DashMpd        0x63
+
+/***************************************************************************/
+/* HDS F4M                                                                 */
+/***************************************************************************/
+
+#define MediaInfo_Parser_HdsF4m         0x64
+
+/***************************************************************************/
+/* DCP Composition Asset Map                                               */
+/***************************************************************************/
+
+#define MediaInfo_Parser_DcpAm          0x65
+
+/***************************************************************************/
+/* DCP Composition Playlist                                                */
+/***************************************************************************/
+
+#define MediaInfo_Parser_DcpCpl         0x66
+
+/***************************************************************************/
+/* DCP Package List                                                        */
+/***************************************************************************/
+
+#define MediaInfo_Parser_DcpPkl         0x67
+
 /***************************************************************************/
 /* MPEG Video                                                              */
 /***************************************************************************/
@@ -459,6 +540,12 @@ struct MediaInfo_Event_DvDif_Analysis_Frame_0
 
 #define MediaInfo_Parser_Avc            0x81
 
+/***************************************************************************/
+/* AVC                                                                     */
+/***************************************************************************/
+
+#define MediaInfo_Parser_Hevc            0x83
+
 /***************************************************************************/
 /* VC-1                                                                    */
 /***************************************************************************/
@@ -477,6 +564,12 @@ struct MediaInfo_Event_DvDif_Analysis_Frame_0
 
 #define MediaInfo_Parser_BarData        0x84
 
+/***************************************************************************/
+/* MPEG-4 Visual                                                           */
+/***************************************************************************/
+
+#define MediaInfo_Parser_Mpeg4v         0x85
+
 /***************************************************************************/
 /* DTS                                                                     */
 /***************************************************************************/
@@ -562,4 +655,46 @@ struct MediaInfo_Event_DvDif_Analysis_Frame_0
 
 #define MediaInfo_Parser_Scte20         0xF5
 
+/***************************************************************************/
+/* DVB Subtitle                                                            */
+/***************************************************************************/
+
+#define MediaInfo_Parser_DvbSubtitle    0xF6
+
+/***************************************************************************/
+/* Teletext                                                                */
+/***************************************************************************/
+
+#define MediaInfo_Parser_Teletext       0xF7
+
+/***************************************************************************/
+/* SCC                                                                     */
+/***************************************************************************/
+
+#define MediaInfo_Parser_Scc            0xF8
+
+/***************************************************************************/
+/* ARIB STD B24/B37                                                        */
+/***************************************************************************/
+
+#define MediaInfo_Parser_AribStdB24B37  0xF9
+
+/***************************************************************************/
+/* TTML                                                                    */
+/***************************************************************************/
+
+#define MediaInfo_Parser_Ttml           0xFA
+
+/***************************************************************************/
+/* SubRip                                                                  */
+/***************************************************************************/
+
+#define MediaInfo_Parser_SubRip         0xFB
+
+/***************************************************************************/
+/* N19                                                                     */
+/***************************************************************************/
+
+#define MediaInfo_Parser_N19            0xFC
+
 #endif //MediaInfo_EventsH
