@@ -1,21 +1,8 @@
-// File_Vc1 - Info for VC-1 files
-// Copyright (C) 2007-2012 MediaArea.net SARL, Info@MediaArea.net
-//
-// This library is free software: you can redistribute it and/or modify it
-// under the terms of the GNU Library General Public License as published by
-// the Free Software Foundation, either version 2 of the License, or
-// any later version.
-//
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-// GNU Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public License
-// along with this library. If not, see <http://www.gnu.org/licenses/>.
-//
-//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
-//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
+/*  Copyright (c) MediaArea.net SARL. All Rights Reserved.
+ *
+ *  Use of this source code is governed by a BSD-style license that can
+ *  be found in the License.html file in the root of the source tree.
+ */
 
 //---------------------------------------------------------------------------
 // Pre-compilation
@@ -48,7 +35,12 @@
 #if MEDIAINFO_EVENTS
     #include "MediaInfo/MediaInfo_Config_MediaInfo.h"
     #include "MediaInfo/MediaInfo_Events.h"
+    #include "MediaInfo/MediaInfo_Events_Internal.h"
 #endif //MEDIAINFO_EVENTS
+#if MEDIAINFO_DEMUX
+    #include <cstring>
+    #include "base64.h"
+#endif //MEDIAINFO_DEMUX
 using namespace std;
 using namespace ZenLib;
 //---------------------------------------------------------------------------
@@ -242,12 +234,35 @@ File_Vc1::File_Vc1()
     EntryPoint_Parsed=false;
     FrameRate=0;
     RefFramesCount=0;
+
+    #if MEDIAINFO_DEMUX
+        InitData_Buffer=NULL;
+        InitData_Buffer_Size=0;
+    #endif //MEDIAINFO_DEMUX
+}
+
+//---------------------------------------------------------------------------
+File_Vc1::~File_Vc1()
+{
+    #if MEDIAINFO_DEMUX
+        delete[] InitData_Buffer;
+    #endif //MEDIAINFO_DEMUX
 }
 
 //***************************************************************************
 // Streams management
 //***************************************************************************
 
+//---------------------------------------------------------------------------
+void File_Vc1::Streams_Accept()
+{
+    //Filling
+    Stream_Prepare(Stream_Video);
+    Fill(Stream_Video, 0, Video_Format, "VC-1");
+    Fill(Stream_Video, 0, Video_Codec, From_WMV3?"WMV3":"VC-1"); //For compatibility with the old reaction
+    Fill(Stream_Video, 0, Video_BitDepth, 8);
+}
+
 //---------------------------------------------------------------------------
 void File_Vc1::Streams_Fill()
 {
@@ -260,12 +275,6 @@ void File_Vc1::Streams_Fill()
     else
         PixelAspectRatio=1; //Unknown
 
-    //Filling
-    Stream_Prepare(Stream_Video);
-    Fill(Stream_Video, 0, Video_Format, "VC-1");
-    Fill(Stream_Video, 0, Video_Codec, From_WMV3?"WMV3":"VC-1"); //For compatibility with the old reaction
-    Fill(Stream_Video, 0, Video_BitDepth, 8);
-
     Ztring Profile;
     if (profile!=(int8u)-1)
         Profile=Vc1_Profile[profile];
@@ -273,6 +282,7 @@ void File_Vc1::Streams_Fill()
         Profile+=__T("@L")+Ztring::ToZtring(level);
     Fill(Stream_Video, 0, Video_Format_Profile, Profile);
     Fill(Stream_Video, 0, Video_Codec_Profile, Profile);
+    Fill(Stream_Video, 0, Video_ColorSpace, "YUV");
     Fill(Stream_Video, 0, Video_Colorimetry, Vc1_ColorimetryFormat[colordiff_format]);
     if (coded_width && coded_height)
     {
@@ -303,7 +313,7 @@ void File_Vc1::Streams_Fill()
     if (Frame_Count>0 && interlace)
         Fill(Stream_Video, 0, Video_ScanOrder, Interlaced_Bottom?"BFF":"TFF");
     std::string TempRef;
-    for (std::map<int16u, temporalreference>::iterator Temp=TemporalReference.begin(); Temp!=TemporalReference.end(); Temp++)
+    for (std::map<int16u, temporalreference>::iterator Temp=TemporalReference.begin(); Temp!=TemporalReference.end(); ++Temp)
     {
         TempRef+=Temp->second.top_field_first?"T":"B";
         TempRef+=Temp->second.repeat_first_field?"3":"2";
@@ -388,15 +398,20 @@ bool File_Vc1::FileHeader_Begin()
 bool File_Vc1::Synched_Test()
 {
     //Must have enough buffer for having header
-    if (Buffer_Offset+3>Buffer_Size)
+    if (Buffer_Offset+4>Buffer_Size)
         return false;
 
     //Quick test of synchro
-    if (CC3(Buffer+Buffer_Offset)!=0x000001)
+    if (Buffer[Buffer_Offset  ]!=0x00
+     || Buffer[Buffer_Offset+1]!=0x00
+     || Buffer[Buffer_Offset+2]!=0x01)
+    {
         Synched=false;
+        return true;
+    }
 
     //Quick search
-    if (Synched && !Header_Parser_QuickSearch())
+    if (!Header_Parser_QuickSearch())
         return false;
 
     #if MEDIAINFO_IBI
@@ -427,7 +442,7 @@ void File_Vc1::Synched_Init()
     frameratecode_dr=0;
     profile=(int8u)-1;
     level=(int8u)-1;
-    colordiff_format=0;
+    colordiff_format=1; //Default is 4:2:0
     AspectRatio=0;
     AspectRatioX=0;
     AspectRatioY=0;
@@ -519,6 +534,40 @@ bool File_Vc1::Demux_UnpacketizeContainer_Test()
             if (Config->Demux_EventWasSent)
                 return false;
         }
+
+        //Demux
+        #if MEDIAINFO_DEMUX
+            if (InitData_Buffer_Size!=(size_t)-1 && Buffer[Buffer_Offset+3]==0x0F) //First SequenceHeader
+            {
+                //Searching begin of frame (after SequenceHeader/EntryPointHeader)
+                size_t Header_End=4;
+                for (; Header_End<Demux_Offset; Header_End++)
+                    if (Buffer[Header_End  ]==0x00
+                     && Buffer[Header_End+1]==0x00
+                     && Buffer[Header_End+2]==0x01
+                     && Buffer[Header_End+2]==0x0D)
+                    break;
+
+                switch (Config->Demux_InitData_Get())
+                {
+                    case 0 :    //In demux event
+                                break; //Will be done in the first demux event
+                    case 1 :    //In field
+                                {
+                                std::string Data_Raw((const char*)(Buffer+Buffer_Offset), (size_t)(Header_End-Buffer_Offset));
+                                std::string Data_Base64(Base64::encode(Data_Raw));
+                                Fill(Stream_Video, StreamPos_Last, "Demux_InitBytes", Data_Base64);
+                                (*Stream_More)[Stream_Video][StreamPos_Last](Ztring().From_Local("Demux_InitBytes"), Info_Options)=__T("N NT");
+                                }
+                                break;
+                    default :   ;
+                }
+
+                delete[] InitData_Buffer; InitData_Buffer=NULL;
+                InitData_Buffer_Size=(size_t)-1;
+            }
+        #endif //MEDIAINFO_DEMUX
+
         Demux_UnpacketizeContainer_Demux(Buffer[Buffer_Offset+3]==0x0F);
     }
 
@@ -651,16 +700,15 @@ bool File_Vc1::Header_Parser_QuickSearch()
         Synched=false;
         if (!Synchronize())
         {
-            if (File_Offset+Buffer_Size==File_Size)
-            {
-                Synched=true;
-                return true;
-            }
+            UnSynched_IsNotJunk=true;
             return false;
         }
 
         if (Buffer_Offset+4>Buffer_Size)
+        {
+            UnSynched_IsNotJunk=true;
             return false;
+        }
     }
 
     if (Buffer_Offset+3==Buffer_Size)
@@ -884,36 +932,25 @@ void File_Vc1::FrameHeader()
 
         #if MEDIAINFO_EVENTS
             {
-                struct MediaInfo_Event_Video_SliceInfo_0 Event;
-                Event.EventCode=MediaInfo_EventCode_Create(MediaInfo_Parser_None, MediaInfo_Event_Video_SliceInfo, 0);
-                Event.Stream_Offset=File_Offset+Buffer_Offset;
-                Event.PCR=FrameInfo.PCR;
-                Event.PTS=FrameInfo.PTS;
-                Event.DTS=FrameInfo.DTS;
-                Event.DUR=FrameInfo.DUR;
-                Event.StreamIDs_Size=StreamIDs_Size;
-                Event.StreamIDs=(MediaInfo_int64u*)StreamIDs;
-                Event.StreamIDs_Width=(MediaInfo_int8u*)StreamIDs_Width;
-                Event.ParserIDs=(MediaInfo_int8u* )ParserIDs;
-                Event.FramePosition=Frame_Count;
-                Event.FieldPosition=Field_Count;
-                Event.SlicePosition=0;
-                switch (ptype)
-                {
-                    case 0 :
-                                Event.SliceType=0; break;
-                    case 1 :
-                                Event.SliceType=1; break;
-                    case 2 :
-                    case 3 :
-                                Event.SliceType=2; break;
-                    case 4 :
-                                Event.SliceType=3; break;
-                    default:
-                                Event.SliceType=(int8u)-1;
-                }
-                Event.Flags=0;
-                Config->Event_Send((const int8u*)&Event, sizeof(MediaInfo_Event_Video_SliceInfo_0));
+                EVENT_BEGIN (Video, SliceInfo, 0)
+                    Event.FieldPosition=Field_Count;
+                    Event.SlicePosition=0;
+                    switch (ptype)
+                    {
+                        case 0 :
+                                    Event.SliceType=0; break;
+                        case 1 :
+                                    Event.SliceType=1; break;
+                        case 2 :
+                        case 3 :
+                                    Event.SliceType=2; break;
+                        case 4 :
+                                    Event.SliceType=3; break;
+                        default:
+                                    Event.SliceType=(int8u)-1;
+                    }
+                    Event.Flags=0;
+                EVENT_END   ()
             }
         #endif //MEDIAINFO_EVENTS
     FILLING_END();
@@ -975,6 +1012,34 @@ void File_Vc1::EntryPointHeader()
         EntryPoint_Parsed=true;
         if (!Status[IsAccepted])
             Accept("VC-1");
+
+        #if MEDIAINFO_DEMUX
+            if (InitData_Buffer_Size && InitData_Buffer_Size!=(size_t)-1)
+            {
+                size_t InitData_Buffer_Temp_Size=InitData_Buffer_Size+(size_t)(Header_Size+Element_Size);
+                int8u* InitData_Buffer_Temp=new int8u[InitData_Buffer_Temp_Size];
+                std::memcpy(InitData_Buffer_Temp, InitData_Buffer, InitData_Buffer_Size);
+                std::memcpy(InitData_Buffer_Temp+InitData_Buffer_Size, Buffer+Buffer_Offset-(size_t)Header_Size, (size_t)(Header_Size+Element_Size));
+
+                switch (Config->Demux_InitData_Get())
+                {
+                    case 0 :    //In demux event
+                                break; //Will be done in the first demux event
+                    case 1 :    //In field
+                                {
+                                std::string Data_Raw((char*)InitData_Buffer_Temp, InitData_Buffer_Temp_Size);
+                                std::string Data_Base64(Base64::encode(Data_Raw));
+                                Fill(Stream_Video, StreamPos_Last, "Demux_InitBytes", Data_Base64);
+                                (*Stream_More)[Stream_Video][StreamPos_Last](Ztring().From_Local("Demux_InitBytes"), Info_Options)=__T("N NT");
+                                }
+                                break;
+                    default :   ;
+                }
+
+                delete[] InitData_Buffer; InitData_Buffer=NULL;
+                InitData_Buffer_Size=(size_t)-1;
+            }
+        #endif //MEDIAINFO_DEMUX
     FILLING_END();
 }
 
@@ -1101,6 +1166,15 @@ void File_Vc1::SequenceHeader()
                 Accept("VC-1");
             Finish("VC-1");
         }
+
+        #if MEDIAINFO_DEMUX
+            if (InitData_Buffer_Size!=(size_t)-1)
+            {
+                InitData_Buffer_Size=(size_t)(Header_Size+Element_Size);
+                InitData_Buffer=new int8u[InitData_Buffer_Size];
+                std::memcpy(InitData_Buffer, Buffer+Buffer_Offset-(size_t)Header_Size, (size_t)(Header_Size+Element_Size));
+            }
+        #endif //MEDIAINFO_DEMUX
     FILLING_END();
 }
 
